/*
 * JNDIELResolver.java
 *
 * Created on July 24, 2006, 10:21 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.sun.faces.extensions.jndi;

import java.beans.FeatureDescriptor;
import java.util.ArrayList;
import java.util.Iterator;

import javax.el.ELContext;
import javax.el.ELException;
import javax.el.ELResolver;
import javax.el.PropertyNotFoundException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.NameClassPair;
import javax.naming.NameNotFoundException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

/**
 * <p>
 * The following special variable names are recognized, and evaluated as
 * indicated:
 * </p>
 * <ul>
 * <li><strong>jndi</strong> - Returns the JNDI context at name
 * <code>java:comp/env</code> (relative to the initial context supplied by the
 * container.</li>
 * </ul>
 * <p>
 * All other evaluations are delegated to <code>ELResolver</code> chain.
 * </p>
 * 
 */
public class JNDIELResolver extends ELResolver {
	/**
	 * <p>
	 * Construct a new {@link ShaleELResolver} instance.
	 * </p>
	 * 
	 */
	public JNDIELResolver() {
	}

	/**
	 * <p>
	 * Variable name to be resoved to our JNDI environment context.
	 * </p>
	 */
	private static final String JNDI_VARIABLE_NAME = "jndi";

	// Prevent returning a huge number of results
	// from getFeatureDescriptor()
	private static final int MAX_TOP_LEVEL_NAMES = 1000;

	// ------------------------------- VariableResolver Methods
	/**
	 * <p>
	 * Resolve variable names known to this resolver; otherwise, delegate to the
	 * ELResolver chain.
	 * </p>
	 * 
	 * @param name
	 *            Variable name to be resolved
	 */
	public Object getValue(ELContext elContext, Object base, Object property) {
		Object result = null;
		// If we have a non-null base object, function as a PropertyResolver
		if (null != base) {
			if (base instanceof Context) {
				elContext.setPropertyResolved(true);
				Context context = (Context) base;
				try {
					if (property instanceof Name) {
						result = context.lookup((Name) property);
					} else {
						if (null != property) {
							result = context.lookup(property.toString());
						}
					}
				} catch (NameNotFoundException e) {
					// Mimic standard JSF/JSP behavior when base is a Map
					// by returning null
					return null;
				} catch (NamingException e) {
					throw new ELException(e);
				}
			}
		} else {
			// function as a VariableResolver
			if (null == property) {
				throw new PropertyNotFoundException(
						"ShaleELResolver: name must " + "not be null");
			}
			if (JNDI_VARIABLE_NAME.equals(property)) {
				elContext.setPropertyResolved(true);
				try {
					InitialContext ic = new InitialContext();
					result = (Context) ic.lookup("java:comp/env");
				} catch (NamingException e) {
					throw new ELException(e);
				}
			}
		}
		return result;
	}

	public Class<?> getType(ELContext elContext, Object base, Object property) {
		if (null != base && base instanceof Context) {
			elContext.setPropertyResolved(true);
			return Object.class;
		}
		return null;
	}

	public void setValue(ELContext elContext, Object base, Object property,
			Object value) {
		if (null != base && base instanceof Context) {
			Context context = (Context) base;
			elContext.setPropertyResolved(true);
			try {
				// Mimic standard JSF/JSP behavior when base is a Map
				// by calling rebind() instead of bind()
				if (property instanceof Name) {
					context.rebind((Name) property, value);
				} else {
					context.rebind(property.toString(), value);
				}
			} catch (NamingException e) {
				throw new ELException(e);
			}
		}
	}

	public boolean isReadOnly(ELContext elContext, Object base, Object property) {
		if (base instanceof Context) {
			elContext.setPropertyResolved(true);
			// Mimic standard JSF/JSP behavior when base is a Map
			// by returning false if we cannot tell any better
			return false;
		}
		return false;
	}

	public Iterator getFeatureDescriptors(ELContext elContext, Object base) {
		String name = null;
		NamingEnumeration<NameClassPair> names = null;
		ArrayList<FeatureDescriptor> featureDescriptors = new ArrayList<FeatureDescriptor>();
		if (null == base) {
			name = "";
		} else {
			name = base.toString();
		}
		try {
			InitialContext ic = new InitialContext();
			Context context = (Context) ic.lookup("java:comp/env");
			NameClassPair cur = null;
			names = context.list(name);
			FeatureDescriptor curDescriptor = null;
			int i = 0;
			while (names.hasMoreElements() && i++ < MAX_TOP_LEVEL_NAMES) {
				cur = names.nextElement();
				curDescriptor = new FeatureDescriptor();
				curDescriptor.setName(cur.getName());
				curDescriptor.setDisplayName(cur.getNameInNamespace());
				curDescriptor.setExpert(true);
				if (null != cur.getClassName()) {
					curDescriptor.setValue(ELResolver.TYPE, Class.forName(cur
							.getClassName()));
				}
				curDescriptor.setValue(ELResolver.RESOLVABLE_AT_DESIGN_TIME,
						true);
				featureDescriptors.add(curDescriptor);
			}
			if (MAX_TOP_LEVEL_NAMES <= i) {
				// log error
			}
		} catch (ClassNotFoundException e) {
			throw new ELException(e);
		} catch (NamingException e) {
			throw new ELException(e);
		}

		return featureDescriptors.iterator();
	}

	public Class<?> getCommonPropertyType(ELContext context, Object base) {
		if (null == base) {
			return Context.class;
		}
		return Object.class;
	}
}